#include <stdlib.h>
#include <stdio.h>

/* 
** Hinweise
***************************************************************************************************/

 /*
  * Dieses Programm dient der Anschauung der Seite
  * http://proggen.org/doku.php?id=c:list:head
  * bzw. als Grundlage, um selbst mit den Quelltexten zu
  * experimentieren.
  * 
  * Am Ende werden nicht alle Nodes freigegeben.
  *
  * Bei Fragen wenden Sie sich einfach ans Forum unter
  * http://forum.proggen.org
  */

/* 
** Strukturen
***************************************************************************************************/

struct Address
{
  char Street[64];
  int  Number;
  int  ZipCode;
  char Town[64];
};
 
struct AddressNode
{
  struct AddressNode * Next;
 
  struct Address       Data;
};

struct AddressList
{
  struct AddressNode * First;
  
  int                  Count;
 
  char               * FileName;
};

/* 
** Deklarationen (Bekanntgabe von Funkionen, die gerufen werden, bevor sie definiert werden)
***************************************************************************************************/

struct AddressNode * GetNode( struct AddressList * list, int index );
int GetIndex( struct AddressList * list, struct AddressNode * node );

/* 
** Konstruktion/Destruktion
***************************************************************************************************/

struct AddressNode * NewNode( struct AddressList * list, struct AddressNode * prevNode )
{
  struct AddressNode * newNode = (struct AddressNode *) malloc( sizeof( struct AddressNode ) );
 
  if( prevNode ) 
  {
    newNode ->Next = prevNode->Next;
    prevNode->Next = newNode;
  }
  else  
  {
    newNode->Next = list->First;
    list->First = newNode;
  }
  
  list->Count++;
 
  printf( "NewNode: Liste enthält jetzt %d Nodes\n", list->Count );

  return newNode;
}

void DeleteNode( struct AddressList * list, struct AddressNode * toBeRemoved )
{
  /* Vorgänger finden */
 
  struct AddressNode * prev;
 
  list->Count--;

  if( toBeRemoved == list->First )
  {
    /* Wir entfernen den Listenkopf, daher beginnt die Liste ab sofort am Nachfolgerelement */
 
    list->First = list->First->Next;          // Listenkopf verschieben

    printf( "DeleteNode (Listenkopf): %d Nodes übrig\n", list->Count );
  }
  else
  {
    /* Es ist nicht das Kopfelement */
 
    int index = GetIndex( list, toBeRemoved );
    prev = GetNode( list, index - 1 );
    prev->Next = toBeRemoved->Next;

    printf( "DeleteNode: %d Nodes übrig\n", list->Count );
  }
 
  free( toBeRemoved );  
}

struct AddressList * NewAddressList()
{
  struct AddressList * newList = (struct AddressList *) malloc( sizeof( struct AddressList ) );
 
  newList->First = NULL;
// newList->Counter = 0;  // Für den Fall, dass ein Counter implementiert wird
 
  return newList;
}

void DeleteAddressList( struct AddressList * toBeRemoved )
{
  /* Alle Elemente entfernen */
 
  while( toBeRemoved->First )
    DeleteNode( toBeRemoved, toBeRemoved->First );
 
  free( toBeRemoved );
}

/* 
** Suchen und Finden
***************************************************************************************************/

int GetIndex( struct AddressList * list, struct AddressNode * element )
{
  int index = 0;
  struct AddressNode * node = list->First;
 
  while( node != element && element != NULL )
  {
    index ++;
 
    node = node->Next;
  }
 
  /* index zurückgeben, wenn gefunden */
 
  if( node == element ) 
    return index;
 
  /* Falls nicht gefunden, Fehler zurückgeben */
 
  return -1;
}

struct AddressNode * GetNode( struct AddressList * list, int index )
{
  struct AddressNode * node = list->First;

  while( index > 0 && node != NULL )
  {
    node = node->Next;
    index--;
  }
 
  return node;
}

/* 
** Hauptprogramm
***************************************************************************************************/

int main( void )
{
  struct AddressList * list;
  struct AddressNode * node;
 
  list = NewAddressList();
 
  node = NewNode( list, NULL );  // Erste Node anlegen.
  node = NewNode( list, node );  // zweite Node anlegen
  node = NewNode( list, node );  // dritte Node anlegen
  NewNode( list, node );         // vierte Node anlegen, Variable 'node' zeigt weiterhin auf 3. Node
 
  /* Node entfernen, auf die 'node' zeigt */
 
  DeleteNode( list, node );
 
  /* Zuvor haben wir uns um die Freigabe des Speichers gar nicht gekümmert! */
 
  DeleteAddressList( list );

  return EXIT_SUCCESS;
}

